/*
Copyright 2016 Cisco Systems, Inc. All rights reserved.

Licensed under the Apache License, Version 2.0 (the "License"); 
you may not use this file except in compliance with the License. 
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, 
software distributed under the License is distributed on an "AS IS"
 BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express 
or implied. See the License for the specific language governing 
permissions and limitations under the License.
*/

.header,.jumbotron{border-bottom:1px solid #e5e5e5}
.jumbotron,input{text-align:center}
.main-content .content h1,body,html{font-weight:100}
.browsehappy{margin:.2em 0;background:#ccc;color:#000;padding:.2em 0}
body{padding-top:60px;padding-bottom:60px;-webkit-font-smoothing:subpixel-antialiased}
.background{position:absolute;left:0;top:0;width:100%;height:100%;background-size:cover;opacity:.1}
.footer,.header,.marketing{padding-left:15px;padding-right:15px}
.header{margin-bottom:10px}
.header h3{margin-top:0;margin-bottom:0;line-height:40px;padding-bottom:19px}
.footer{padding-top:19px;color:#777;border-top:1px solid #e5e5e5}
.container-narrow>hr{margin:30px 0}
.jumbotron .btn{font-size:21px;padding:14px 24px}
.marketing{margin:40px 0}
.marketing p+h4{margin-top:28px}
@media screen and (min-width:768px){.container{max-width:1280px}
.footer,.header,.marketing{padding-left:0;padding-right:0}
.header{margin-bottom:30px}
.jumbotron{border-bottom:0}
}
.input-group-addon{min-width:150px}
[ng-message]{text-align:left;padding-right:0}
md-input-container{padding-bottom:5px}
div[ng-messages]{min-height:5px}
td.ng-binding.ng-scope{padding:3px}
.panel-randread{border-color:#a6bddb}
.panel-randread>.panel-heading{color:#fff;background-color:#a6bddb;border-color:#a6bddb}
.panel-randwrite{border-color:#74a9cf}
.panel-randwrite>.panel-heading{color:#fff;background-color:#74a9cf;border-color:#74a9cf}
.panel-randrw{border-color:#3690c0}
.panel-randrw>.panel-heading{color:#fff;background-color:#3690c0;border-color:#3690c0}
.panel-read{border-color:#feb24c}
.panel-read>.panel-heading{color:#fff;background-color:#feb24c;border-color:#feb24c}
.panel-write{border-color:#fd8d3c}
.panel-write>.panel-heading{color:#fff;background-color:#fd8d3c;border-color:#fd8d3c}
.panel-rw{border-color:#fc4e2a}
.panel-rw>.panel-heading{color:#fff;background-color:#fc4e2a;border-color:#fc4e2a}
:focus,button,input,select,textarea{outline:0}
#loading-bar-spinner{top:70px}
#loading-bar .bar{top:60px;height:5px}
#loading-bar-spinner .spinner-icon{width:20px;height:20px}
.pause{-webkit-animation-play-state:paused!important;-moz-animation-play-state:paused!important;-o-animation-play-state:paused!important;animation-play-state:paused!important}
.container{position:relative;height:100%;width:100%;left:0;-webkit-transition:left .4s ease-in-out;-moz-transition:left .4s ease-in-out;-ms-transition:left .4s ease-in-out;-o-transition:left .4s ease-in-out;transition:left .4s ease-in-out}
#sidebar,.swipe-area{position:absolute;height:100%}
.container .open-sidebar{left:300px}
.swipe-area{width:50px;left:0;top:0;z-index:0}
#sidebar{width:300px;left:-300px;box-sizing:border-box;-moz-box-sizing:border-box}
#sidebar ul{margin:0;padding:0;list-style:none}
#sidebar ul li{margin:0}
#sidebar ul li a{padding:15px 20px;font-size:16px;font-weight:100;color:#fff;text-decoration:none;display:block;-webkit-transition:background .3s ease-in-out;-moz-transition:background .3s ease-in-out;-ms-transition:background .3s ease-in-out;-o-transition:background .3s ease-in-out;transition:background .3s ease-in-out}
#content.content-smaller{width:70%;height:100%}
.main-content{width:100%;height:100%;padding:10px;box-sizing:border-box;-moz-box-sizing:border-box;position:relative}
.main-content .content{box-sizing:border-box;-moz-box-sizing:border-box;padding-left:60px;width:100%}
.main-content .content p{width:100%;line-height:160%}
.main-content #sidebar-toggle{background:#DF314D;border-radius:3px;display:block;position:relative;padding:10px 7px;float:left}
.main-content #sidebar-toggle .bar{display:block}
.supercontainer{position:relative;height:100%;width:100%;left:0;overflow:hidden}
.loader,.loader:after,.loader:before{background:#fff;-webkit-animation:load1 1s infinite ease-in-out;animation:load1 1s infinite ease-in-out;width:1em;height:4em}
.loader:after,.loader:before{position:absolute;top:0;content:''}
.loader,.loadera{position:relative;text-indent:-9999em}
.loader:before{left:-1.5em;-webkit-animation-delay:-.32s;animation-delay:-.32s}
.loader{margin:6px auto;font-size:3px;-webkit-transform:translateZ(0);-ms-transform:translateZ(0);transform:translateZ(0);-webkit-animation-delay:-.16s;animation-delay:-.16s}
.loader:after{left:1.5em}
@-webkit-keyframes load1{0%,100%,80%{box-shadow:0 0 #fff;height:4em}
40%{box-shadow:0 -2em #fff;height:5em}
}
@keyframes load1{0%,100%,80%{box-shadow:0 0 #fff;height:4em}
40%{box-shadow:0 -2em #fff;height:5em}
}
.loadera,.loadera:after,.loadera:before{border-radius:50%;width:2.5em;height:2.5em;-webkit-animation:load7 1.8s infinite ease-in-out;animation:load7 1.8s infinite ease-in-out}
.loadera{font-size:2px;margin:15px 5px;-webkit-transform:translateZ(0);-ms-transform:translateZ(0);transform:translateZ(0);-webkit-animation-delay:-.16s;animation-delay:-.16s}
.loadera:before{left:-3.5em;-webkit-animation-delay:-.32s;animation-delay:-.32s}
.loadera:after{left:3.5em}
.loadera:after,.loadera:before{content:'';position:absolute;top:0}
@-webkit-keyframes load7{0%,100%,80%{box-shadow:0 2.5em 0 -1.3em #fff}
40%{box-shadow:0 2.5em 0 0 #fff}
}
@keyframes load7{0%,100%,80%{box-shadow:0 2.5em 0 -1.3em #fff}
40%{box-shadow:0 2.5em 0 0 #fff}
}
@keyframes loading{0%,100%,50%{transform:rotate(0)}
25%,75%{transform:rotate(160deg)}
}
.loading{background-color:transparent;border-radius:50%;margin:2px auto;animation:loading 5s infinite linear}
.spin-2,.spin-3,.spin-4,.spin-5,.spin-6{animation-delay:1s}
.spin-1{border:2px solid #c00353;border-top:2px solid transparent;border-left:2px solid transparent;width:48px;height:48px}
.spin-2{border:2px solid #05bbc3;border-top:2px solid transparent;border-left:2px solid transparent;width:40px;height:40px}
.spin-3{border:2px solid #d2d947;border-top:2px solid transparent;border-left:2px solid transparent;width:32px;height:32px}
.spin-4{border:2px solid #f2a342;border-top:2px solid transparent;border-left:2px solid transparent;width:24px;height:24px}
.spin-5{border:2px solid #f34d51;border-top:2px solid transparent;border-left:2px solid transparent;width:16px;height:16px}
.spin-6{border:2px solid #000;width:8px;height:8px}
.js .inputfile{width:.1px;height:.1px;opacity:0;overflow:hidden;position:absolute;z-index:-1}
.inputfile+label{max-width:80%;font-size:1.25rem;font-weight:700;text-overflow:ellipsis;white-space:nowrap;cursor:pointer;display:inline-block;overflow:hidden;padding:.625rem 1.25rem}
.inputfile.has-focus+label,.inputfile:focus+label{outline:#000 dotted 1px;outline:-webkit-focus-ring-color auto 5px}
.inputfile+label svg{width:1em;height:1em;vertical-align:middle;fill:currentColor;margin-top:-.25em;margin-right:.25em}
.inputfile-3+label{color:#acb6c0}
.inputfile-3+label:hover,.inputfile-3.has-focus+label,.inputfile-3:focus+label{color:rgba(70,70,70,.85)}
.dialogdemoBasicUsage #popupContainer{position:relative}
.dialogdemoBasicUsage .footer{width:100%;text-align:center;margin-left:20px}
.dialogdemoBasicUsage .footer,.dialogdemoBasicUsage .footer>code{font-size:.8em;margin-top:50px}