<!--
Copyright 2016 Cisco Systems, Inc. All rights reserved.

Licensed under the Apache License, Version 2.0 (the "License"); 
you may not use this file except in compliance with the License. 
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, 
software distributed under the License is distributed on an "AS IS"
 BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express 
or implied. See the License for the specific language governing 
permissions and limitations under the License.
-->

<div class="col-lg-14">
<div class="well bs-component">
<form class="form-horizontal">
<fieldset>
<table class="table table-striped table-hover">
<thead>
<tr>
<th>
<span class="col-md-2" style="float:left;line-height:25px; height:5px">LOGS</span>
<md-radio-group ng-model="logNum" ng-click="getLog()">
<div class="col-md-2">
<md-radio-button value="0" class="md-primary" style="margin: 0 0">Show All Logs</md-radio-button>
</div>
<div class="col-md-3">
<md-radio-button value="100" class="md-primary" style="margin: 0 0"> Show Last 100 Lines
</md-radio-button>
</div>
</md-radio-group>
<span style="float:right">
<button type="submit" class="btn btn-default btn-xs" ng-click="delLog()" style="width: 65px">Clear
</button>
<button type="submit" class="btn btn-primary btn-xs" ng-click="getLog()" style="width: 65px">Refresh
</button>
</span>
</th>
</tr>
</thead>
</table>
<div id="cc" style="font-family:Courier New, Monospace;width:100%;overflow: auto;word-wrap: break-word; white-space:pre ;font-size:13px"></div>
</fieldset>
</form>
</div>
</div>